﻿using System;
using System.Collections.Generic;
using System.Collections.ObjectModel;
using System.Windows;
using ThinkGeo.MapSuite.Core;
using ThinkGeo.MapSuite.WpfDesktopEdition;
using System.Windows.Input;
using System.Diagnostics;


namespace SampleApp_MS3_7583
{
    /// <summary>
    /// Interaction logic for TestWindow.xaml
    /// </summary>
    public partial class TestWindow : Window
    {
        //public static readonly RectangleShape MaxExtentForWrappingDateline = new RectangleShape(-400767557376, 20026376, 400767557376, -20026376);

        public  int TileSize = 382;
        public const int DotsPerInch = 96;
        public const double MetersInInch = 39.3701;
        public const double DecimalDegreeInInch = 4374754;
        private ManagedProj4Projection proj4;

        private ManagedProj4Projection defaultProj;
        private ShapeFileFeatureLayer worldLayer;
        private LayerOverlay worldOverlay;
        private LayerOverlay overlay;
        private InMemoryFeatureLayer featureLayer;
        private PopupOverlay popupOverlay;

        public TestWindow()
        {
            InitializeComponent();
        }

        private void Window_Loaded(object sender, RoutedEventArgs e)
        {
            BaseShape.GeometryLibrary = GeometryLibrary.Managed;

            wpfMap1.MapUnit = GeographyUnit.Meter;
            wpfMap1.BackgroundOverlay.BackgroundBrush = new GeoSolidBrush(GeoColor.FromArgb(255, 198, 255, 255));

            proj4 = new MyManagedProj4Projection();
            proj4.InternalProjectionParametersString = ManagedProj4Projection.GetWgs84ParametersString();
            proj4.ExternalProjectionParametersString = ManagedProj4Projection.GetSphericalMercatorParametersString();
            proj4.Open();

            defaultProj = new ManagedProj4Projection();
            defaultProj.InternalProjectionParametersString = ManagedProj4Projection.GetWgs84ParametersString();
            defaultProj.ExternalProjectionParametersString = ManagedProj4Projection.GetSphericalMercatorParametersString();
            defaultProj.Open();

            worldLayer = new ShapeFileFeatureLayer(@"..\..\Data\Countries02.shp");
            worldLayer.ZoomLevelSet.ZoomLevel01.DefaultAreaStyle = AreaStyles.Country1;
            worldLayer.ZoomLevelSet.ZoomLevel01.ApplyUntilZoomLevel = ApplyUntilZoomLevel.Level20;
            worldLayer.FeatureSource.Projection = proj4;
            worldLayer.Open();

            var wrappingExtent = worldLayer.GetBoundingBox();
            worldOverlay = new LayerOverlay();
            worldLayer.WrappingExtent = wrappingExtent;
            worldLayer.WrappingMode = WrappingMode.WrapDateline;
            worldOverlay.WrappingMode = WrappingMode.WrapDateline;
            worldOverlay.WrappingExtent = wrappingExtent;
            worldOverlay.Layers.Add("worldLayer", worldLayer);
            wpfMap1.Overlays.Add("WorldOverlay", worldOverlay);



         
            GraticuleAdornmentLayer graticuleAdornmentLayer = new GraticuleAdornmentLayer();
            graticuleAdornmentLayer.GraticuleLineStyle = LineStyles.DegreeLine1;
            graticuleAdornmentLayer.WrappingMode = WrappingMode.WrapDateline;
            graticuleAdornmentLayer.WrappingExtent = wrappingExtent;
            graticuleAdornmentLayer.Projection = defaultProj;
            wpfMap1.AdornmentOverlay.Layers.Add("graticule", graticuleAdornmentLayer);


             worldOverlay.TileCache = GetTileCache(wrappingExtent);
             SetCustomZoomLevels(wrappingExtent);


             LayerOverlay pointsOverlay = new LayerOverlay();
             pointsOverlay.WrappingExtent = wrappingExtent;
             pointsOverlay.TileType = TileType.SingleTile;
             pointsOverlay.WrappingMode = WrappingMode.WrapDateline;
             pointsOverlay.Layers.Add(CreatePointsLayer());
             wpfMap1.Overlays.Add("pointsOverlay", pointsOverlay);



            SetMapExtent();

            wpfMap1.MapResizeMode = MapResizeMode.PreserveExtent;

            var watch = Stopwatch.StartNew();
            wpfMap1.Refresh();
            Debug.WriteLine("Refresh Time (sec): " + watch.Elapsed.TotalSeconds);
        }

        private InMemoryFeatureLayer CreatePointsLayer()
        {
            
            InMemoryFeatureLayer pointsLayer = new InMemoryFeatureLayer();
            pointsLayer.WrappingExtent = worldLayer.GetBoundingBox();
            pointsLayer.WrappingMode = WrappingMode.WrapDateline;
            pointsLayer.ZoomLevelSet.ZoomLevel01.DefaultPointStyle = new MyPointStyle(@"..\..\Data\icon.png", wpfMap1.ZoomLevelSet);
            pointsLayer.ZoomLevelSet.ZoomLevel01.ApplyUntilZoomLevel = ApplyUntilZoomLevel.Level20;
            int fromZoomLevel = 0;

            var watch = Stopwatch.StartNew();
            for (double i = -180; i < 180; i += 0.5)
            {
                for (double n = -90; n < 90; n += 0.5)
                {
                    PointShape pointShape = new PointShape(i, n);


                    if (i % 8 == 0 && n % 8 == 0)
                    {
                        fromZoomLevel = 1;
                    }
                    else if (i % 5 == 0 && n % 5 == 0)
                    {
                        fromZoomLevel = 2;
                    }
                    else if (i % 3 == 0 && n % 3 == 0)
                    {
                        fromZoomLevel = 3;
                    }
                    else if (i % 2 == 0 && n % 2 == 0)
                    {
                        fromZoomLevel = 4;
                    }
                    else
                    {
                        fromZoomLevel = 6;
                    }

                    double angle = new Random().NextDouble() * 360;
                    Feature feature = new Feature(pointShape);
                    //feature.ColumnValues.Add("zoomLevel", fromZoomLevel.ToString());
                    //feature.ColumnValues.Add("angle", angle.ToString());
                    feature.Tag = angle + "#" + fromZoomLevel;
                    pointsLayer.InternalFeatures.Add(feature);
                }
            }

            Debug.WriteLine("Filling InternalFeatures (sec): " + watch.Elapsed.TotalSeconds);
            pointsLayer.FeatureSource.Projection = proj4;
            return pointsLayer;

           
        }

        private BitmapTileCache GetTileCache(RectangleShape bbox)
        {
            FileBitmapTileCache bitmapTileCache = new FileBitmapTileCache();

         
      
            bitmapTileCache.ClearCache();
           
            bitmapTileCache.CacheDirectory = AppDomain.CurrentDomain.BaseDirectory + @"Resources\Tiles\";
            bitmapTileCache.CacheId = "Mercator";
            bitmapTileCache.TileAccessMode = TileAccessMode.ReadAddDelete;
            bitmapTileCache.ImageFormat = TileImageFormat.Png;
            return bitmapTileCache;
        }

      
   
        private void SetCustomZoomLevels(RectangleShape bbox)
        {

           
            double screenWidth = wpfMap1.ActualWidth;
            int tileSize=-1;
            for (double i = 5; i <= 10; i++)
            {
                if (screenWidth % i == 0)
                {
                    tileSize = (int)(screenWidth / i);
                    break;
                }
            }
            if (tileSize == -1)
            {
                for (double i = 4; i <= 2; i--)
                {
                    if (screenWidth % i == 0)
                    {
                        tileSize = (int)(screenWidth / i);
                        break;
                    }
                }
            }

            if (tileSize == -1)
            {
                for (double i = 11; i <= 20; i++)
                {
                    if (screenWidth % i == 0)
                    {
                        tileSize = (int)(screenWidth / i);
                        break;
                    }
                }
            }
            if (tileSize == -1)
            {
                tileSize = (int)Math.Floor(screenWidth);
            }

            TileSize = tileSize;

            int drawWidth = Convert.ToInt32(Math.Floor(screenWidth / TileSize)) * TileSize;
            double resolution = bbox.Width / drawWidth;
            double scaleMax = resolution * MetersInInch * DotsPerInch;
            double scaleMax1 = scaleMax;
            for (int i = 0; i < 15; i++)
            {
                wpfMap1.ZoomLevelSet.CustomZoomLevels.Add(new ZoomLevel(scaleMax));
                scaleMax = scaleMax / 2;
            }

            worldOverlay.TileHeight = TileSize;
            worldOverlay.TileWidth = TileSize;

            MapSuiteTileMatrix tileMatrix = new MapSuiteTileMatrix(scaleMax1, TileSize, TileSize, GeographyUnit.Meter, bbox);
            worldOverlay.TileCache.TileMatrix = tileMatrix;

        }

        private void SetMapExtent()
        {
            RectangleShape rectDegree = new RectangleShape(-140, 80, 140, -75);
            RectangleShape rectMeters = proj4.ConvertToExternalProjection(rectDegree);

            wpfMap1.CurrentExtent = rectMeters;
            wpfMap1.RestrictExtent = new RectangleShape(-20037376 * 10, 15538711.0963092, 20037376 * 10, -12932243.111992);
        }

   
      

    
  
    }

    class MyManagedProj4Projection : ManagedProj4Projection
    {
        protected override Vertex[] ConvertToInternalProjectionCore(double[] x, double[] y)
        {
            for (int i = 0; i < x.Length; i++)
            {

                if (x[i] < -20037508.2314698)
                {

                    x[i] = -20037508.2314698;

                }

                if (x[i] > 20037508.2314698)
                {

                    x[i] = 20037508.2314698;

                }

            }

            return base.ConvertToInternalProjectionCore(x, y);

        }
    }
}